@@ -2,17 +2,18 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import unittest
-import time
 import datetime
 from itertools import combinations
 
+from trytond.model.exceptions import SQLConstraintError
 from trytond.tests.test_tryton import activate_module, with_transaction
 from trytond.tests.trigger import TRIGGER_LOGS
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond.exceptions import UserError
 from trytond.pyson import PYSONEncoder, Eval
 
+from trytond.ir.exceptions import TriggerConditionError
+
 
 class TriggerTestCase(unittest.TestCase):
     'Test Trigger'
@@ -21,6 +22,17 @@ class TriggerTestCase(unittest.TestCase):
     def setUpClass(cls):
         activate_module('tests')
 
+    def setUp(self):
+        TRIGGER_LOGS.clear()
+
+    def run_tasks(self):
+        pool = Pool()
+        Queue = pool.get('ir.queue')
+        transaction = Transaction()
+        while transaction.tasks:
+            task = Queue(transaction.tasks.pop())
+            task.run()
+
     @with_transaction()
     def test_constraints(self):
         'Test constraints'
@@ -32,19 +44,15 @@ class TriggerTestCase(unittest.TestCase):
         model, = Model.search([
                 ('model', '=', 'test.triggered'),
                 ])
-        action_model, = Model.search([
-                ('model', '=', 'test.trigger_action'),
-                ])
 
         values = {
             'name': 'Test',
             'model': model.id,
             'on_time': True,
             'condition': 'true',
-            'action_model': action_model.id,
-            'action_function': 'test',
+            'action': 'test.trigger_action|trigger',
             }
-        self.assert_(Trigger.create([values]))
+        self.assertTrue(Trigger.create([values]))
 
         transaction.rollback()
 
@@ -55,14 +63,14 @@ class TriggerTestCase(unittest.TestCase):
                 combination_values = values.copy()
                 for mode in combination:
                     combination_values['on_%s' % mode] = True
-                self.assertRaises(UserError, Trigger.create,
+                self.assertRaises(SQLConstraintError, Trigger.create,
                     [combination_values])
                 transaction.rollback()
 
         # check_condition
         condition_values = values.copy()
         condition_values['condition'] = '='
-        self.assertRaises(UserError, Trigger.create,
+        self.assertRaises(TriggerConditionError, Trigger.create,
             [condition_values])
         transaction.rollback()
 
@@ -80,23 +88,20 @@ class TriggerTestCase(unittest.TestCase):
         model, = Model.search([
                 ('model', '=', 'test.triggered'),
                 ])
-        action_model, = Model.search([
-                ('model', '=', 'test.trigger_action'),
-                ])
 
         trigger, = Trigger.create([{
                     'name': 'Test',
                     'model': model.id,
                     'on_create': True,
                     'condition': 'true',
-                    'action_model': action_model.id,
-                    'action_function': 'trigger',
+                    'action': 'test.trigger_action|trigger',
                     }])
 
         triggered, = Triggered.create([{
                     'name': 'Test',
                     }])
 
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
@@ -111,6 +116,7 @@ class TriggerTestCase(unittest.TestCase):
         triggered, = Triggered.create([{
                     'name': 'Bar',
                     }])
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
@@ -118,6 +124,7 @@ class TriggerTestCase(unittest.TestCase):
         triggered, = Triggered.create([{
                     'name': 'Foo',
                     }])
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [])
 
         # With limit number
@@ -128,6 +135,7 @@ class TriggerTestCase(unittest.TestCase):
         triggered, = Triggered.create([{
                     'name': 'Test',
                     }])
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
@@ -139,6 +147,7 @@ class TriggerTestCase(unittest.TestCase):
         triggered, = Triggered.create([{
                     'name': 'Test',
                     }])
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
@@ -146,27 +155,24 @@ class TriggerTestCase(unittest.TestCase):
         Trigger._get_triggers_cache.clear()
 
     @with_transaction()
-    def test0030on_write(self):
+    def test_on_write(self):
         'Test on_write'
         pool = Pool()
         Model = pool.get('ir.model')
         Trigger = pool.get('ir.trigger')
+        TriggerLog = pool.get('ir.trigger.log')
         Triggered = pool.get('test.triggered')
 
         model, = Model.search([
                 ('model', '=', 'test.triggered'),
                 ])
-        action_model, = Model.search([
-                ('model', '=', 'test.trigger_action'),
-                ])
 
         trigger, = Trigger.create([{
                     'name': 'Test',
                     'model': model.id,
                     'on_write': True,
                     'condition': 'true',
-                    'action_model': action_model.id,
-                    'action_function': 'trigger',
+                    'action': 'test.trigger_action|trigger',
                     }])
 
         triggered, = Triggered.create([{
@@ -176,6 +182,7 @@ class TriggerTestCase(unittest.TestCase):
         Triggered.write([triggered], {
                 'name': 'Foo',
                 })
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [])
 
         # Trigger with condition
@@ -189,6 +196,7 @@ class TriggerTestCase(unittest.TestCase):
         Triggered.write([triggered], {
                 'name': 'Bar',
                 })
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
@@ -196,12 +204,14 @@ class TriggerTestCase(unittest.TestCase):
         Triggered.write([triggered], {
                 'name': 'Bar',
                 })
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [])
 
         # Different change in condition
         Triggered.write([triggered], {
                 'name': 'Foo',
                 })
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [])
 
         # With limit number
@@ -223,6 +233,7 @@ class TriggerTestCase(unittest.TestCase):
         Triggered.write([triggered], {
                 'name': 'Bar',
                 })
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
@@ -238,6 +249,7 @@ class TriggerTestCase(unittest.TestCase):
             Triggered.write([triggered], {
                     'name': name,
                     })
+        self.run_tasks()
         self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
@@ -249,22 +261,33 @@ class TriggerTestCase(unittest.TestCase):
                     }])
         for name in ('Bar', 'Foo'):
             Triggered.write([triggered], {
-                    'name': name,
-                    })
-        time.sleep(1.2)
+                'name': name,
+                })
+            self.run_tasks()
+        self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
+        TRIGGER_LOGS.pop()
+        Transaction().trigger_records.clear()
+
+        # Make time pass by moving back in time the log creation
+        trigger_log = TriggerLog.__table__()
+        cursor = Transaction().connection.cursor()
+        cursor.execute(*trigger_log.update(
+                [trigger_log.create_date],
+                [datetime.datetime.now() - datetime.timedelta(days=1)],
+                where=trigger_log.record_id == triggered.id))
+
         Triggered.write([triggered], {
                 'name': 'Bar',
                 })
-        self.assertEqual(TRIGGER_LOGS,
-            [([triggered], trigger), ([triggered], trigger)])
-        TRIGGER_LOGS.pop()
+        self.run_tasks()
+        self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
         # Restart the cache on the get_triggers method of ir.trigger
         Trigger._get_triggers_cache.clear()
 
     @with_transaction()
-    def test0040on_delete(self):
+    def test_on_delete(self):
         'Test on_delete'
         pool = Pool()
         Model = pool.get('ir.model')
@@ -275,9 +298,6 @@ class TriggerTestCase(unittest.TestCase):
         model, = Model.search([
                 ('model', '=', 'test.triggered'),
                 ])
-        action_model, = Model.search([
-                ('model', '=', 'test.trigger_action'),
-                ])
 
         triggered, = Triggered.create([{
                     'name': 'Test',
@@ -288,8 +308,7 @@ class TriggerTestCase(unittest.TestCase):
                     'model': model.id,
                     'on_delete': True,
                     'condition': 'true',
-                    'action_model': action_model.id,
-                    'action_function': 'trigger',
+                    'action': 'test.trigger_action|trigger',
                     }])
 
         Triggered.delete([triggered])
@@ -370,24 +389,20 @@ class TriggerTestCase(unittest.TestCase):
         model, = Model.search([
                 ('model', '=', 'test.triggered'),
                 ])
-        action_model, = Model.search([
-                ('model', '=', 'test.trigger_action'),
-                ])
 
         trigger, = Trigger.create([{
                     'name': 'Test',
                     'model': model.id,
                     'on_time': True,
                     'condition': 'true',
-                    'action_model': action_model.id,
-                    'action_function': 'trigger',
+                    'action': 'test.trigger_action|trigger',
                     }])
 
         triggered, = Triggered.create([{
                     'name': 'Test',
                     }])
         Trigger.trigger_time()
-        self.assert_(TRIGGER_LOGS == [([triggered], trigger)])
+        self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
         # Trigger with condition
@@ -402,7 +417,7 @@ class TriggerTestCase(unittest.TestCase):
                 'name': 'Bar',
                 })
         Trigger.trigger_time()
-        self.assert_(TRIGGER_LOGS == [([triggered], trigger)])
+        self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
         # Non matching condition
@@ -410,7 +425,7 @@ class TriggerTestCase(unittest.TestCase):
                 'name': 'Foo',
                 })
         Trigger.trigger_time()
-        self.assert_(TRIGGER_LOGS == [])
+        self.assertEqual(TRIGGER_LOGS, [])
 
         # With limit number
         Trigger.write([trigger], {
@@ -419,7 +434,7 @@ class TriggerTestCase(unittest.TestCase):
                 })
         Trigger.trigger_time()
         Trigger.trigger_time()
-        self.assert_(TRIGGER_LOGS == [([triggered], trigger)])
+        self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
 
         # Delete trigger logs of limit number test
@@ -434,7 +449,7 @@ class TriggerTestCase(unittest.TestCase):
                 })
         Trigger.trigger_time()
         Trigger.trigger_time()
-        self.assert_(TRIGGER_LOGS == [([triggered], trigger)])
+        self.assertEqual(TRIGGER_LOGS, [([triggered], trigger)])
         TRIGGER_LOGS.pop()
         Transaction().delete = {}
 
@@ -447,10 +462,18 @@ class TriggerTestCase(unittest.TestCase):
                 'minimum_time_delay': datetime.timedelta(seconds=1),
                 })
         Trigger.trigger_time()
-        time.sleep(1.2)
+
+        # Make time pass by moving back in time the log creation
+        trigger_log = TriggerLog.__table__()
+        cursor = Transaction().connection.cursor()
+        cursor.execute(*trigger_log.update(
+                [trigger_log.create_date],
+                [datetime.datetime.now() - datetime.timedelta(days=1)],
+                where=trigger_log.record_id == triggered.id))
+
         Trigger.trigger_time()
-        self.assert_(TRIGGER_LOGS == [([triggered], trigger),
-                ([triggered], trigger)])
+        self.assertEqual(
+            TRIGGER_LOGS, [([triggered], trigger), ([triggered], trigger)])
         TRIGGER_LOGS.pop()
         TRIGGER_LOGS.pop()
         Transaction().delete = {}
